<?php

namespace Drupal\commerce_ajaxify\Form;

use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\HtmlCommand;
use Drupal\commerce_product\Entity\Product;
use Drupal\commerce_cart\CartProviderInterface;
use Drupal\commerce_cart\CartManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

class AjaxAddRemoveCartForm extends FormBase {

    /**
     * The current user.
     *
     * @var \Drupal\Core\Session\AccountInterface
     */
    protected $account;

    /**
     * The cart manager.
     *
     * @var \Drupal\commerce_cart\CartManagerInterface
     */
    protected $cartManager;

    /**
     * The cart provider.
     *
     * @var \Drupal\commerce_cart\CartProviderInterface
     */
    protected $cartProvider;

    /**
     * Constructs a new CartController object.
     *
     * @param \Drupal\commerce_cart\CartProviderInterface $cart_provider
     *   The cart provider.
     */
    public function __construct(AccountInterface $account, CartManagerInterface $cart_manager, CartProviderInterface $cart_provider) {
        $this->account = $account;
        $this->cartManager = $cart_manager;
        $this->cartProvider = $cart_provider;
    }

    /**
     * {@inheritdoc}
     */
    public static function create(ContainerInterface $container) {
        return new static(
                $container->get('current_user'),
                $container->get('commerce_cart.cart_manager'),
                $container->get('commerce_cart.cart_provider')
        );
    }

    public function getFormId() {
        return 'ca_add_remove_cart_form';
    }

    public function buildForm(array $form, FormStateInterface $form_state, $product_id = NULL) {

        $form['#attached']['library'][] = 'commerce_ajaxify/commerce_ajaxify';

        if( \Drupal::request()->request->get('product_id') ) {
            $product_id = (int)\Drupal::request()->request->get('product_id');
        }

        $product = Product::load((int) $product_id);

        /* Load Product Variations */
        $entity_manager = \Drupal::entityTypeManager();
        $arr_product_variation = $entity_manager->getStorage('commerce_product_variation')->loadMultiple(array_values($product->getVariationIds()));
        $list_product_variation = [];
        foreach ($arr_product_variation as $product_variation) {
            $list_product_variation[$product_variation->Id()] = $product_variation->getTitle();
        }
        $form['product_variation']['#type'] = 'select';
        $form['product_variation']['#options'] = $list_product_variation;

        $form['product_id']['#type'] = 'hidden';
        $form['product_id']['#default_value'] = $product_id;

        $storeId = $product->get('stores')->getValue()[0]['target_id'];

        $store = \Drupal::entityTypeManager()
                ->getStorage('commerce_store')
                ->load($storeId);

        $order = $this->cartProvider->getCart('default', $store, $this->account);

        if($order) {
          $items = $order->getItems();

          $quantity = FALSE;
          $bool_remove = FALSE;
          foreach ($items as $item) {
            foreach ($arr_product_variation as $product_variation) {
              if ($item->getPurchasedEntity()->Id() == $product_variation->Id()) {
                $form['product_variation']['#default_value'] = $product_variation->Id();
                $quantity = (int)$item->getQuantity();
//                    \Drupal::logger('remove')->error('In remove');
                $bool_remove = TRUE;
                break;
              }
            }
          }
        }
        if( isset($form_state->getTriggeringElement()['#name']) && $form_state->getTriggeringElement()['#name'] == 'remove') {
            if($quantity - 1) {
                \Drupal::logger('remove')->error('In remove');
                $bool_remove = TRUE;
            } else {
                \Drupal::logger('remove')->error('In 2remove');
                $bool_remove = FALSE;
            }
        }
        if($bool_remove) {
            $form['remove'] = [
                '#name' => 'remove',
                '#type' => 'button',
                '#value' => $this->t('Remove'),
                '#attributes' => [
                    'class' => [
                        'btn',
                        'btn-md',
                        'btn-primary',
                        'use-ajax-submit'
                    ]
                ],
                '#ajax' => [
                    'callback' => '::removeFromCart',
                ]
            ];

        }
        $form['remove']['#prefix'] = '<div class="result_message-' . $product_id . '">';
        $str_quantity = ($quantity) ? $quantity : '';
        $form['remove']['#suffix'] = '<div class="quantity">' . $str_quantity . '</div></div>';

        $form['submit'] = [
            '#type' => 'submit',
            '#value' => $this->t('Add'),
            '#attributes' => [
                'class' => [
                    'btn',
                    'btn-md',
                    'btn-primary',
                    'use-ajax-submit'
                ]
            ],
            '#ajax' => [
                'callback' => '::addToCart',
            ]
        ];

        return $form;
    }

    public function validateForm(array &$form, FormStateInterface $form_state) {
        parent::validateForm($form, $form_state);
    }

    public function submitForm(array &$form, FormStateInterface $form_state) {

    }

    public function addToCart(array $form, FormStateInterface $form_state) {

        $product_id = $form_state->getValues()['product_id'];

        $productObj = Product::load($product_id);

        $product_variation_id = $form_state->getValues()['product_variation'];
        $storeId = $productObj->get('stores')->getValue()[0]['target_id'];
        $variationobj = \Drupal::entityTypeManager()
                ->getStorage('commerce_product_variation')
                ->load($product_variation_id);
        $store = \Drupal::entityTypeManager()
                ->getStorage('commerce_store')
                ->load($storeId);

        $cart = $this->cartProvider->getCart('default', $store);

        if (!$cart) {
            $cart = $this->cartProvider->createCart('default', $store);
        }

        $line_item_type_storage = \Drupal::entityTypeManager()
                ->getStorage('commerce_order_item_type');
        $cart_manager = \Drupal::service('commerce_cart.cart_manager');
        $cart_manager->addEntity($cart, $variationobj);

        $order = $this->cartProvider->getCart('default', $store, $this->account);

        $items = $order->getItems();
        $quantity = FALSE;
        $total_item_in_cart = 0;

        foreach ($items as $item) {
            if ($item->getPurchasedEntity()->Id() == $product_variation_id) {
                $quantity = (int)$item->getQuantity();
                break;
            }
        }
        foreach ($items as $item) {
            $total_item_in_cart +=  $item->getQuantity();
        }


        $response = new AjaxResponse();

        if($quantity) {
            $str_quantity = ($quantity) ? $quantity : '';
            $form['remove']['#suffix'] = '<div class="quantity">' . $str_quantity . '</div>';
            $response->addCommand(
                new HtmlCommand(
                        '.result_message-' . $product_id,
                        $form['remove']
                )
            );
        }

        $response->addCommand(
                new HtmlCommand(
                        '.cart-block--summary__count',
                        $total_item_in_cart . ' ' . t('items')
                )
        );
        return $response;
    }

    public function removeFromCart(array $form, FormStateInterface $form_state) {

        $product_id = $form_state->getValues()['product_id'];

        if( \Drupal::request()->query->get('product_id') ) {
            $product_id = (int)\Drupal::request()->query->get('product_id');
        }

        $productObj = Product::load($product_id);

        $product_variation_id = $form_state->getValues()['product_variation'];

        if( \Drupal::request()->query->get('product_variation') ) {
            $product_variation_id = (int)\Drupal::request()->query->get('product_variation');
        }

        $storeId = $productObj->get('stores')->getValue()[0]['target_id'];

        $store = \Drupal::entityTypeManager()
                ->getStorage('commerce_store')
                ->load($storeId);

        $order = $this->cartProvider->getCart('default', $store, $this->account);

        $items = $order->getItems();
        $total_item_in_cart = 0;
        $item_quantity = 0;

        foreach ($items as $item) {
            $quantity = $item->getQuantity();
            if ($item->getPurchasedEntity()->Id() == $product_variation_id) {
                $quantity = $quantity - 1;
                if ($quantity > 0) {
                    $item->setQuantity($quantity);
                    $this->cartManager->updateOrderItem($order, $item);
                } else {
                    $this->cartManager->removeOrderItem($order, $item);
                }
                $item_quantity = $quantity;
            }
            $total_item_in_cart += $quantity;
        }



        $response = new AjaxResponse();
        if($item_quantity) {
            $str_quantity = ($item_quantity) ? $item_quantity : '';
            $form['remove']['#suffix'] = '<div class="quantity">' . (int) $str_quantity . '</div>';
            $response->addCommand(
                new HtmlCommand(
                        '.result_message-' . $product_id,
                        $form['remove']
                )
            );
        }

        $response->addCommand(
                new HtmlCommand(
                        '.cart-block--summary__count',
                        $total_item_in_cart . ' ' . t('items')
                )
        );
        return $response;
    }

}
